@@ -23,9 +23,10 @@ TODOs:
 
 import numpy as N
 
+from mvpa import _random_seed
 
 # Rely on SG
-from mvpa.base import externals
+from mvpa.base import externals, warning
 if externals.exists('shogun', raiseException=True):
     import shogun.Features
     import shogun.Classifier
@@ -33,16 +34,37 @@ if externals.exists('shogun', raiseException=True):
     import shogun.Kernel
     import shogun.Library
 
+    # Figure out debug IDs once and for all
+    if hasattr(shogun.Kernel, 'M_DEBUG'):
+        _M_DEBUG = shogun.Kernel.M_DEBUG
+        _M_ERROR = shogun.Kernel.M_ERROR
+    elif hasattr(shogun.Kernel, 'MSG_DEBUG'):
+        _M_DEBUG = shogun.Kernel.MSG_DEBUG
+        _M_ERROR = shogun.Kernel.MSG_ERROR
+    else:
+        _M_DEBUG, _M_ERROR = None, None
+        warning("Could not figure out debug IDs within shogun. "
+                "No control over shogun verbosity would be provided")
+
+    try:
+        # reuse the same seed for shogun
+        shogun.Library.Math_init_random(_random_seed)
+        # and do it twice to stick ;) for some reason is necessary
+        # atm
+        shogun.Library.Math_init_random(_random_seed)
+    except Exception, e:
+        warning('Shogun cannot be seeded due to %s' % (e,))
+
 import operator
 
 from mvpa.misc.param import Parameter
 from mvpa.base import warning
 
+from mvpa.clfs.base import FailedToTrainError
 from mvpa.clfs.meta import MulticlassClassifier
 from mvpa.clfs._svmbase import _SVM
 from mvpa.misc.state import StateVariable
 from mvpa.measures.base import Sensitivity
-from mvpa.base import externals
 
 from sens import *
 
@@ -50,8 +72,6 @@ if __debug__:
     from mvpa.base import debug
 
 
-
-
 def _setdebug(obj, partname):
     """Helper to set level of debugging output for SG
     :Parameters:
@@ -61,10 +81,12 @@ def _setdebug(obj, partname):
         For what kind of object we are talking about... could be automated
         later on (TODO)
     """
+    if _M_DEBUG is None:
+        return
     debugname = "SG_%s" % partname.upper()
 
-    switch = {True: (shogun.Kernel.M_DEBUG, 'M_DEBUG', "enable"),
-              False: (shogun.Kernel.M_ERROR, 'M_ERROR', "disable")}
+    switch = {True: (_M_DEBUG, 'M_DEBUG', "enable"),
+              False: (_M_ERROR, 'M_ERROR', "disable")}
 
     key = __debug__ and debugname in debug.active
 
@@ -183,8 +205,6 @@ class SVM(_SVM):
             "libsvr": (shogun.Regression.LibSVR, ('C', 'tube_epsilon',),
                       ('regression',),
                        "LIBSVM's epsilon-SVR"),
-            "krr": (shogun.Regression.KRR, ('tau',), ('regression',),
-                    "Kernel Ridge Regression"),
             }
 
 
@@ -222,7 +242,13 @@ class SVM(_SVM):
         #     but then krr gets confused, and svrlight needs it to provide
         #     meaningful results even without 'retraining'
         if self._svm_impl in ['svrlight', 'lightsvm']:
-            kernel.set_precompute_matrix(True, True)
+            try:
+                kernel.set_precompute_matrix(True, True)
+            except Exception, e:
+                # N/A in shogun 0.9.1... TODO: RF
+                if __debug__:
+                    debug('SG_', "Failed call to set_precompute_matrix for %s: %s"
+                          % (self, e))
 
 
     def _train(self, dataset):
@@ -259,7 +285,8 @@ class SVM(_SVM):
                 # assure that we have -1/+1
                 _labels_dict = {ul[0]:-1.0, ul[1]:+1.0}
             elif len(ul) < 2:
-                raise ValueError, "we do not have 1-class SVM brought into SG yet"
+                raise FailedToTrainError, \
+                      "We do not have 1-class SVM brought into SG yet"
             else:
                 # can't use plain enumerate since we need them swapped
                 _labels_dict = dict([ (ul[i], i) for i in range(len(ul))])
@@ -384,6 +411,14 @@ class SVM(_SVM):
             else:
                 self.__svm = svm_impl_class(Cs[0], self.__kernel, labels)
                 self.__svm.set_epsilon(self.params.epsilon)
+
+            # Set shrinking
+            if self.params.isKnown('shrinking'):
+                shrinking = self.params.shrinking
+                if __debug__:
+                    debug("SG_", "Setting shrinking to %s" % shrinking)
+                self.__svm.set_shrinking_enabled(shrinking)
+
             if Cs is not None and len(Cs) == 2:
                 if __debug__:
                     debug("SG_", "Since multiple Cs are provided: %s, assign them" % Cs)
@@ -647,7 +682,10 @@ for name, item, params, descr in \
          ('lightsvm', "shogun.Classifier.SVMLight", "('C',), ('binary',)",
           "SVMLight classification http://svmlight.joachims.org/"),
          ('svrlight', "shogun.Regression.SVRLight", "('C','tube_epsilon',), ('regression',)",
-          "SVMLight regression http://svmlight.joachims.org/")]:
+          "SVMLight regression http://svmlight.joachims.org/"),
+         ('krr', "shogun.Regression.KRR", "('tau',), ('regression',)",
+          "Kernel Ridge Regression"),
+         ]:
     if externals.exists('shogun.%s' % name):
         exec "SVM._KNOWN_IMPLEMENTATIONS[\"%s\"] = (%s, %s, \"%s\")" % (name, item, params, descr)
 
